@@ -45,7 +45,7 @@ function(object,...){
     digits        <- 4
   }
   cat("\nMean Kullback-Leibler\ndiscrepancy calibration (KLDC):\n")
-  if (!is.null(object$K)){
+  if (object$K[1] != "Not calculated") {
     mean.q        <- (object$K$q12 + object$K$q21) / 2
     print.default(mean.q, digits = digits)
   } else {
@@ -53,7 +53,8 @@ function(object,...){
       cat("KLDC was not calculated due to insufficient number",
           " of simulations to estimate convergence.\n")
     } else {
-      cat("KLDC was not calculated due to lack of convergence,","or because covariates were not included in the model.\n")
+      cat("KLDC was not calculated due to lack of convergence,",
+          "or because covariates were not included in the model.\n")
     }
   }
   
@@ -62,28 +63,40 @@ function(object,...){
   print.default(object$coefficients, digits = digits)
 
   cat("\nConvergence:\n")
-  if (is.null(object$DIC)){
+  if (object$Conv[1] == "Not calculated") {
     if (object$set['nsim'] == 1) {
       cat("\nConvergence calculations require more than one run.",
-          "\nTo estimate potential scale reduction run at least two simulations.\n")
+          "\nTo estimate potential scale reduction run at least",
+          "two simulations.\n")
     } else {
       cat("\nWarning: Convergence not reached for some parameters",
-          " (i.e. 'PotScaleReduc' values larger than 1.1).",
+          " (i.e. 'PotScaleReduc' values larger than 1.15).",
           "\nThese estimates should not be used for inference.\n")
     }
   } else {
-    cat("Appropriate convergence reached for all parameters.\n")
+    if (all(object$Conv[, "Rhat"] < 1.15)) {
+      cat("Appropriate convergence reached for all parameters.\n")
+    } else {
+      cat("\nWarning: Convergence not reached for some parameters",
+          " (i.e. 'PotScaleReduc' values larger than 1.15).",
+          "\nThese estimates should not be used for inference.\n")
+    }
   } 
   cat("\nDIC:\n")
-  if (!is.null(object$DIC)){
+  if (object$DIC[1] != "Not calculated"){
     cat(object$DIC["DIC"],"\n")
   } else {
     if (object$set['nsim'] == 1) {
       cat("DIC was not calculated due to insufficient number",
-          " of simulations to estimate convergence.\n")
+          "of simulations to estimate convergence.\n")
     } else {
       cat("DIC was not calculated due to lack of convergence.\n")
     }
   }
+  ans <- c(list(coefficients = object$coef, DIC = object$DIC,
+          KullbackLeibler = object$KullbackLeibler, 
+          Convergence = object$Convergence,
+          modelSpecs = object$ModelSpecs, settings = object$set))
+  return(invisible(ans))
 }
 
